! ***********************************************************************
!
!   Copyright (C) 2010  Bill Paxton
!
!   this file is part of mesa.
!
!   mesa is free software; you can redistribute it and/or modify
!   it under the terms of the gnu general library public license as published
!   by the free software foundation; either version 2 of the license, or
!   (at your option) any later version.
!
!   mesa is distributed in the hope that it will be useful, 
!   but without any warranty; without even the implied warranty of
!   merchantability or fitness for a particular purpose.  see the
!   gnu library general public license for more details.
!
!   you should have received a copy of the gnu library general public license
!   along with this software; if not, write to the free software
!   foundation, inc., 59 temple place, suite 330, boston, ma 02111-1307 usa
!
! ***********************************************************************
 
module run_star_extras

  use star_lib
  use star_def
  use const_def

  implicit none
      
  integer, save, private  :: i_write = 0
      
contains
      
  subroutine extras_controls(id, ierr)

    integer, intent(in) :: id
    integer, intent(out) :: ierr
    type (star_info), pointer :: s

    ierr = 0

  end subroutine extras_controls
  
!****    
      
  integer function extras_startup(id, restart, ierr)

    integer, intent(in) :: id
    logical, intent(in) :: restart
    integer, intent(out) :: ierr

    type (star_info), pointer :: s

    ierr = 0

    call star_ptr(id, s, ierr)
    if (ierr /= 0) return

    extras_startup = 0
    if (.not. restart) then
       call alloc_extra_info(s)
    else ! it is a restart
       call unpack_extra_info(s)
    end if

  end function extras_startup

!****

  integer function extras_check_model(id, id_extra)

    use core_hgroup

    integer, intent(in) :: id, id_extra
    integer :: ierr

    type (star_info), pointer :: s

    ierr = 0

    call star_ptr(id, s, ierr)
    if (ierr /= 0) return

    ! Finish

    extras_check_model = keep_going

  end function extras_check_model

!****

  integer function how_many_extra_history_columns(id, id_extra)

    integer, intent(in) :: id, id_extra

    how_many_extra_history_columns = 0

  end function how_many_extra_history_columns

!****    
      
  subroutine data_for_extra_history_columns(id, id_extra, n, names, vals, ierr)

    integer, intent(in) :: id, id_extra, n
    character (len=maxlen_history_column_name) :: names(n)
    real(dp) :: vals(n)
    integer, intent(out) :: ierr

    ierr = 0

  end subroutine data_for_extra_history_columns

!****

  integer function how_many_extra_profile_columns(id, id_extra)

    use star_def, only: star_info
    integer, intent(in) :: id, id_extra
    integer :: ierr
    type (star_info), pointer :: s

    how_many_extra_profile_columns = 0

  end function how_many_extra_profile_columns

!****      
      
  subroutine data_for_extra_profile_columns(id, id_extra, n, nz, names, vals, ierr)

    use star_def, only: star_info, maxlen_profile_column_name
    use const_def, only: dp

    integer, intent(in) :: id, id_extra, n, nz
    character (len=maxlen_profile_column_name) :: names(n)
    real(dp) :: vals(nz,n)
    integer, intent(out) :: ierr

    ierr = 0

  end subroutine data_for_extra_profile_columns

!****

  integer function extras_finish_step(id, id_extra)

    integer, intent(in) :: id, id_extra

    integer                   :: ierr
    type (star_info), pointer :: s
    logical, save             :: first = .true.
    real(dp), save            :: logT_prev
    real(dp), save            :: logL_prev
    real(dp)                  :: dlgTeff
    real(dp)                  :: dlgL
    real(dp), save            :: dHR

    ierr = 0

    call star_ptr(id, s, ierr)
    if (ierr /= 0) return

    extras_finish_step = keep_going
    call store_extra_info(s)

    ! to save a profile, 
    ! s% need_to_save_profiles_now = .true.
    ! to update the star log,
    ! s% need_to_update_history_now = .true.

    ! Update the track parameter

    if (first) then

       dHR = 0.

    else

       dlgTeff = LOG10(s%Teff) - logT_prev
       dlgL = LOG10(s%L_phot) - logL_prev

       dHR = dHR + SQRT((s%delta_HR_ds_Teff*dlgTeff)**2 + (s%delta_HR_ds_L*dlgL)**2)

    endif

    logT_prev = LOG10(s%Teff)
    logL_prev = LOG10(s%L_phot)

    ! Decide whether to write

    if (s%pulse_info_format == 'GSM') then
       if (first) then
          i_write = 0
          call write_gsm_model(id, ierr)
          first = .FALSE.
       elseif (dHR >= 1.) then
          i_write = i_write + 1
          call write_gsm_model(id, ierr)
          dHR = dHR - INT(dHR)
       endif
    endif

  end function extras_finish_step

!****

  subroutine extras_after_evolve(id, id_extra, ierr)

    integer, intent(in) :: id, id_extra
    integer, intent(out) :: ierr
    type (star_info), pointer :: s

    ierr = 0

    call star_ptr(id, s, ierr)
    if (ierr /= 0) return

  end subroutine extras_after_evolve

!****

  subroutine alloc_extra_info(s)
    integer, parameter :: extra_info_alloc = 1
    type (star_info), pointer :: s
    call move_extra_info(s,extra_info_alloc)
  end subroutine alloc_extra_info

  subroutine unpack_extra_info(s)
    integer, parameter :: extra_info_get = 2
    type (star_info), pointer :: s
    call move_extra_info(s,extra_info_get)
  end subroutine unpack_extra_info

  subroutine store_extra_info(s)
    integer, parameter :: extra_info_put = 3
    type (star_info), pointer :: s
    call move_extra_info(s,extra_info_put)
  end subroutine store_extra_info

  subroutine move_extra_info(s,op)
    integer, parameter :: extra_info_alloc = 1
    integer, parameter :: extra_info_get = 2
    integer, parameter :: extra_info_put = 3
    type (star_info), pointer :: s
    integer, intent(in) :: op

    integer :: i, j, num_ints, num_dbls, ierr

    i = 0
    ! call move_int or move_flg    
    num_ints = i

    i = 0
    ! call move_dbl       

    num_dbls = i

    if (op /= extra_info_alloc) return
    if (num_ints == 0 .and. num_dbls == 0) return

    ierr = 0
    call star_alloc_extras(s% id, num_ints, num_dbls, ierr)
    if (ierr /= 0) then
       write(*,*) 'failed in star_alloc_extras'
       write(*,*) 'alloc_extras num_ints', num_ints
       write(*,*) 'alloc_extras num_dbls', num_dbls
       stop 1
    end if

  contains

    subroutine move_dbl(dbl)
      real(dp) :: dbl
      i = i+1
      select case (op)
      case (extra_info_get)
         dbl = s% extra_work(i)
      case (extra_info_put)
         s% extra_work(i) = dbl
      end select
    end subroutine move_dbl

    subroutine move_int(int)
      integer :: int
      i = i+1
      select case (op)
      case (extra_info_get)
         int = s% extra_iwork(i)
      case (extra_info_put)
         s% extra_iwork(i) = int
      end select
    end subroutine move_int

    subroutine move_flg(flg)
      logical :: flg
      i = i+1
      select case (op)
      case (extra_info_get)
         flg = (s% extra_iwork(i) /= 0)
      case (extra_info_put)
         if (flg) then
            s% extra_iwork(i) = 1
         else
            s% extra_iwork(i) = 0
         end if
      end select
    end subroutine move_flg

  end subroutine move_extra_info

!****

  subroutine write_gsm_model (id, ierr)

    use core_hgroup

    integer, intent(in)  :: id
    integer, intent(out) :: ierr

    type(star_info), pointer :: s
    real(dp) :: M_star
    real(dp) :: R_star
    real(dp) :: L_star
    real(dp), dimension(:), pointer :: &
         r, w, L, p, rho, T, &
         N2, Gamma_1, nabla_ad, delta, nabla,  &
         kappa, kappa_rho, kappa_T, &
         epsilon, epsilon_rho, epsilon_T, omega
    character(256) :: filename
    type(hgroup_t) :: hg
    integer        :: n

    ierr = 0

    call star_ptr(id, s, ierr)
    if (ierr /= 0) return

    ! Get model data

    call star_get_gyre_info(id, .TRUE., .FALSE., &
         M_star, R_star, L_star, r, w, L, p, rho, T, &
            N2, Gamma_1, nabla_ad, delta, nabla,  &
            kappa, kappa_rho, kappa_T, &
            epsilon, epsilon_rho, epsilon_T, omega, ierr)
    if (ierr /= 0) return

    ! Write it to a GSM-format file

    write(filename, 100) i_write
100 format('model-', I4.4, '.h5')

    print *,'Writing GSM model to '//TRIM(filename)

    hg = hgroup_t(filename, CREATE_FILE)

    ! Write the header

    n = SIZE(r)

    call write_attr(hg, 'n', n)

    call write_attr(hg, 'M_star', M_star)
    call write_attr(hg, 'R_star', R_star)
    call write_attr(hg, 'L_star', L_star)

    ! Write additional fields (not strictly part of the GSM format)

    call write_attr(hg, 'model_number', s%model_number)

    associate (n_c => s%num_mixing_regions)

      if (n_c > 0) then
         if (s%largest_conv_mixing_region == n_c .AND. &
             s%mixing_region_bottom(n_c) == s%nz .AND. &
             s%mixing_region_type(n_c) == 1) then
            call write_attr(hg, 'x_core', (s%conv_mx1_top_r*rsol)/R_star)
         else
            call write_attr(hg, 'x_core', (s%he_core_radius*rsol)/R_star)
         endif
      else
         call write_attr(hg, 'x_core', (s%he_core_radius*rsol)/R_star)
      endif

    end associate
    
    ! Write the data

    call write_dset(hg, 'r', r)
    call write_dset(hg, 'w', w)
    call write_dset(hg, 'p', p)
    call write_dset(hg, 'rho', rho)
    call write_dset(hg, 'T', T)
    call write_dset(hg, 'N2', N2)
    call write_dset(hg, 'Gamma_1', Gamma_1)
    call write_dset(hg, 'nabla_ad', nabla_ad)
    call write_dset(hg, 'delta', delta)
    call write_dset(hg, 'nabla', nabla)
    call write_dset(hg, 'epsilon', epsilon)
    call write_dset(hg, 'epsilon_rho', epsilon_rho)
    call write_dset(hg, 'epsilon_T', epsilon_T)
    call write_dset(hg, 'kappa', kappa)
    call write_dset(hg, 'kappa_rho', kappa_rho)
    call write_dset(hg, 'kappa_T', kappa_T)
    call write_dset(hg, 'Omega_rot', omega)

    call hg%final()

    ! Deallocate arrays

    deallocate(r)
    deallocate(w)
    deallocate(p)
    deallocate(rho)
    deallocate(T)
    deallocate(N2)
    deallocate(Gamma_1)
    deallocate(nabla_ad)
    deallocate(delta)
    deallocate(nabla)
    deallocate(epsilon)
    deallocate(epsilon_rho)
    deallocate(epsilon_T)
    deallocate(kappa)
    deallocate(kappa_rho)
    deallocate(kappa_T)
    deallocate(omega)

    ! Finish

  end subroutine write_gsm_model

end module run_star_extras

